@@ -11,7 +11,7 @@ from typing import Tuple
 
 from simplyblock_core import utils, constants, distr_controller
 from simplyblock_core.controllers import snapshot_controller, pool_controller, lvol_events, caching_node_controller
-from simplyblock_core.kv_store import DBController
+from simplyblock_core.db_controller import DBController
 from simplyblock_core.models.nvme_device import NVMeDevice
 from simplyblock_core.models.pool import Pool
 from simplyblock_core.models.lvol_model import LVol
@@ -107,7 +107,7 @@ def validate_add_lvol_func(name, size, host_id_or_name, pool_id_or_name,
     #  pool validation
     pool = None
     for p in db_controller.get_pools():
-        if pool_id_or_name == p.id or pool_id_or_name == p.pool_name:
+        if pool_id_or_name == p.get_id() or pool_id_or_name == p.pool_name:
             pool = p
             break
     if not pool:
@@ -187,10 +187,11 @@ def _get_next_3_nodes(cluster_id, lvol_size=0):
             #     continue
             #
             online_nodes.append(node)
+            lvols = db_controller.get_lvols_by_node_id(node.get_id()) or []
             # node_stat_list = db_controller.get_node_stats(node, limit=1000)
             # combined_record = utils.sum_records(node_stat_list)
             node_st = {
-                "lvol": len(node.lvols)+1,
+                "lvol": len(lvols)+1,
                 # "cpu": 1 + (node.cpu * node.cpu_hz),
                 # "r_io": combined_record.read_io_ps,
                 # "w_io": combined_record.write_io_ps,
@@ -294,7 +295,8 @@ def validate_aes_xts_keys(key1: str, key2: str) -> Tuple[bool, str]:
 
 def add_lvol_ha(name, size, host_id_or_name, ha_type, pool_id_or_name, use_comp, use_crypto,
                 distr_vuid, max_rw_iops, max_rw_mbytes, max_r_mbytes, max_w_mbytes,
-                with_snapshot=False, max_size=0, crypto_key1=None, crypto_key2=None, lvol_priority_class=0):
+                with_snapshot=False, max_size=0, crypto_key1=None, crypto_key2=None, lvol_priority_class=0,
+                uid=None, pvc_name=None, namespace=None):
 
     logger.info(f"Adding LVol: {name}")
     host_node = None
@@ -307,7 +309,7 @@ def add_lvol_ha(name, size, host_id_or_name, ha_type, pool_id_or_name, use_comp,
 
     pool = None
     for p in db_controller.get_pools():
-        if pool_id_or_name == p.id or pool_id_or_name == p.pool_name:
+        if pool_id_or_name == p.get_id() or pool_id_or_name == p.pool_name:
             pool = p
             break
     if not pool:
@@ -396,6 +398,8 @@ def add_lvol_ha(name, size, host_id_or_name, ha_type, pool_id_or_name, use_comp,
     logger.info(f"Max size: {utils.humanbytes(max_size)}")
     lvol = LVol()
     lvol.lvol_name = name
+    lvol.pvc_name = pvc_name or ""
+    lvol.namespace = namespace or ""
     lvol.size = int(size)
     lvol.max_size = int(max_size)
     lvol.status = LVol.STATUS_ONLINE
@@ -403,7 +407,7 @@ def add_lvol_ha(name, size, host_id_or_name, ha_type, pool_id_or_name, use_comp,
     lvol.create_dt = str(datetime.now())
     lvol.ha_type = ha_type
     lvol.bdev_stack = []
-    lvol.uuid = str(uuid.uuid4())
+    lvol.uuid = uid or str(uuid.uuid4())
     lvol.guid = _generate_hex_string(16)
     lvol.vuid = vuid
     lvol.lvol_bdev = f"LVOL_{vuid}"
@@ -451,7 +455,7 @@ def add_lvol_ha(name, size, host_id_or_name, ha_type, pool_id_or_name, use_comp,
     }
 
     if lvol.lvol_priority_class:
-        lvol_dict["lvol_priority_class"] = lvol.lvol_priority_class
+        lvol_dict["params"]["lvol_priority_class"] = lvol.lvol_priority_class
     lvol.bdev_stack = [lvol_dict]
 
     if use_crypto:
@@ -520,14 +524,14 @@ def add_lvol_ha(name, size, host_id_or_name, ha_type, pool_id_or_name, use_comp,
             host_node.secondary_node_id]
         lvol.nodes = nodes_ids
 
-    host_node = db_controller.get_storage_node_by_id(host_node.get_id())
-    host_node.lvols.append(lvol.uuid)
-    host_node.write_to_db(db_controller.kv_store)
+    # host_node = db_controller.get_storage_node_by_id(host_node.get_id())
+    # host_node.lvols += 1
+    # host_node.write_to_db(db_controller.kv_store)
 
-    pool = db_controller.get_pool_by_id(pool.get_id())
-    lvol.pool_uuid = pool.id
-    pool.lvols.append(lvol.uuid)
-    pool.write_to_db(db_controller.kv_store)
+    # pool = db_controller.get_pool_by_id(pool.get_id())
+    lvol.pool_uuid = pool.get_id()
+    # pool.lvols += 1
+    # pool.write_to_db(db_controller.kv_store)
 
     lvol.write_to_db(db_controller.kv_store)
     lvol_events.lvol_create(lvol)
@@ -567,6 +571,13 @@ def _create_bdev_stack(lvol, snode):
             else:
                 ret = rpc_client.create_lvol(**params)
 
+        elif type == "bdev_lvol_clone":
+            if snode.is_secondary_node:
+                ret = rpc_client.bdev_lvol_clone_register(
+                    lvol.lvol_bdev, lvol.snapshot_name, lvol.lvol_uuid, lvol.blobid)
+            else:
+                ret = rpc_client.lvol_clone(**params)
+
         else:
             logger.debug(f"Unknown BDev type: {type}")
             continue
@@ -598,7 +609,8 @@ def add_lvol_on_node(lvol, snode, ha_comm_addrs=None, ha_inode_self=0):
         huge_free = result["memory_details"]["huge_free"]
 
         total_node_capacity = db_controller.get_snode_size(snode.get_id())
-        error = utils.validate_add_lvol_or_snap_on_node(memory_free, huge_free, snode.max_lvol, lvol.size,  total_node_capacity, len(snode.lvols))
+        lvols = len(db_controller.get_lvols_by_node_id(snode.get_id()))
+        error = utils.validate_add_lvol_or_snap_on_node(memory_free, huge_free, snode.max_lvol, lvol.size,  total_node_capacity, lvols)
         if error:
             logger.error(error)
             return False, f"Failed to add lvol on node {snode.get_id()}"
@@ -661,15 +673,29 @@ def add_lvol_on_node(lvol, snode, ha_comm_addrs=None, ha_inode_self=0):
 def recreate_lvol_on_node(lvol, snode, ha_inode_self=0, ana_state=None):
     rpc_client = RPCClient(snode.mgmt_ip, snode.rpc_port, snode.rpc_username, snode.rpc_password)
 
-    lv = rpc_client.get_bdevs(f"{lvol.lvs_name}/{lvol.lvol_bdev}")
-    if not lv:
-        return False, "LVol bdev not found!"
+    base=f"{lvol.lvs_name}/{lvol.lvol_bdev}"
+    retry = 3
+    while retry > 0:
+        lv = rpc_client.get_bdevs(base)
+        if lv:
+            break
+        else:
+            retry -= 1
+            msg = f"LVol bdev not found: {base} on node {snode.get_id()}, retrying"
+            logger.warning(msg)
+            time.sleep(2)
+    else:
+        msg = f"LVol bdev not found: {base} on node {snode.get_id()}"
+        logger.error(msg)
+        return False, msg
 
     if "crypto" in lvol.lvol_type:
         ret = _create_crypto_lvol(
-            rpc_client, lvol.crypto_bdev, f"{lvol.lvs_name}/{lvol.lvol_bdev}", lvol.crypto_key1, lvol.crypto_key2)
+            rpc_client, lvol.crypto_bdev, base, lvol.crypto_key1, lvol.crypto_key2)
         if not ret:
-            return False, "Failed to create crypto bdev"
+            msg=f"Failed to create crypto lvol on node {snode.get_id()}"
+            logger.error(msg)
+            return False, msg
 
     namespace_found = False
     subsys_found = False
@@ -693,7 +719,6 @@ def recreate_lvol_on_node(lvol, snode, ha_inode_self=0, ana_state=None):
         if not ret:
             return False, "Failed to add bdev to subsystem"
 
-    cluster = db_controller.get_cluster_by_id(snode.cluster_id)
     # add listeners
     logger.info("adding listeners")
     for iface in snode.data_nics:
@@ -906,18 +931,16 @@ def delete_lvol(id_or_name, force_delete=False):
                 return False
 
     # remove from db
-    snode = db_controller.get_storage_node_by_id(lvol.node_id)
-    # logger.debug(snode)
-    logger.debug(f"removing lvol: {lvol.get_id()} from node lvol list: {str(snode.lvols)}")
-    if lvol.get_id() in snode.lvols:
-        snode.lvols.remove(lvol.get_id())
-        snode.write_to_db(db_controller.kv_store)
+    # snode = db_controller.get_storage_node_by_id(lvol.node_id)
+    # # logger.debug(snode)
+    # logger.debug(f"removing lvol: {lvol.get_id()} from node {snode.get_id()}")
+    # snode.lvols -= 1
+    # snode.write_to_db(db_controller.kv_store)
 
-    # remove from pool
-    pool = db_controller.get_pool_by_id(lvol.pool_uuid)
-    if lvol.get_id() in pool.lvols:
-        pool.lvols.remove(lvol.get_id())
-        pool.write_to_db(db_controller.kv_store)
+    # # remove from pool
+    # pool = db_controller.get_pool_by_id(lvol.pool_uuid)
+    # pool.lvols -= 1
+    # pool.write_to_db(db_controller.kv_store)
 
     lvol_events.lvol_delete(lvol)
     lvol.remove(db_controller.kv_store)
@@ -997,16 +1020,16 @@ def list_lvols(is_json, cluster_id, pool_id_or_name, all=False):
         if not pool:
             pool = db_controller.get_pool_by_name(pool_id_or_name)
             if pool:
-                for lv_id in pool.lvols:
-                    lvols.append(db_controller.get_lvol_by_id(lv_id))
+                for lv in db_controller.get_lvols_by_pool_id(pool.get_id()):
+                    lvols.append(lv)
     else:
-        lvols = db_controller.get_lvols()
+        lvols = db_controller.get_all_lvols()
 
     data = []
     for lvol in lvols:
         logger.debug(lvol)
-        if lvol.deleted is True and all is False:
-            continue
+        # if lvol.deleted is True and all is False:
+        #     continue
         data.append({
             "Id": lvol.uuid,
             "Name": lvol.lvol_name,
@@ -1097,9 +1120,13 @@ def connect_lvol(uuid):
                 "ip": ip,
                 "port": port,
                 "nqn": lvol.nqn,
+                "reconnect-delay": constants.LVOL_NVME_CONNECT_RECONNECT_DELAY,
+                "ctrl-loss-tmo": constants.LVOL_NVME_CONNECT_CTRL_LOSS_TMO,
+                "nr-io-queues": constants.LVOL_NVME_CONNECT_NR_IO_QUEUES,
                 "connect": f"sudo nvme connect --reconnect-delay={constants.LVOL_NVME_CONNECT_RECONNECT_DELAY} "
-                           f"--ctrl-loss-tmo={constants.LVOL_NVME_CONNECT_CTRL_LOSS_TMO} --transport={transport} "
-                           f"--traddr={ip} --trsvcid={port} --nqn={lvol.nqn}",
+                           f"--ctrl-loss-tmo={constants.LVOL_NVME_CONNECT_CTRL_LOSS_TMO} "
+                           f"--nr-io-queues={constants.LVOL_NVME_CONNECT_NR_IO_QUEUES} "
+                           f"--transport={transport} --traddr={ip} --trsvcid={port} --nqn={lvol.nqn}",
             })
     return out
 
@@ -1127,7 +1154,7 @@ def resize_lvol(id, new_size):
         logger.error(f"Can not resize clone!")
         return False
 
-    logger.info(f"Resizing LVol: {lvol.id}, new size: {new_size}")
+    logger.info(f"Resizing LVol: {lvol.get_id()}, new size: {new_size}")
 
     snode = db_controller.get_storage_node_by_id(lvol.node_id)
 
@@ -1161,7 +1188,7 @@ def set_read_only(id):
         logger.error(f"Pool is disabled")
         return False
 
-    logger.info(f"Setting LVol: {lvol.id} read only")
+    logger.info(f"Setting LVol: {lvol.get_id()} read only")
 
     snode = db_controller.get_storage_node_by_hostname(lvol.hostname)
 
@@ -1304,8 +1331,8 @@ def migrate(lvol_id, node_id):
                 return ret
         lvol.nodes = nodes_ids
 
-    host_node.lvols.append(lvol.uuid)
-    host_node.write_to_db(db_controller.kv_store)
+    # host_node.lvols.append(lvol.uuid)
+    # host_node.write_to_db(db_controller.kv_store)
     lvol.write_to_db(db_controller.kv_store)
 
     lvol_events.lvol_migrate(lvol, old_node_id, lvol.node_id)
@@ -1348,8 +1375,8 @@ def move(lvol_id, node_id, force=False):
                     delete_lvol_from_node(lvol_id, nodes_id, clear_data=False)
 
             # remove from storage node
-            src_node.lvols.remove(lvol_id)
-            src_node.write_to_db(db_controller.kv_store)
+            # src_node.lvols.remove(lvol_id)
+            # src_node.write_to_db(db_controller.kv_store)
         return True
     else:
         logger.error("Failed to migrate lvol")
@@ -1370,7 +1397,7 @@ def inflate_lvol(lvol_id):
         logger.error(f"Pool is disabled")
         return False
 
-    logger.info(f"Inflating LVol: {lvol.id}")
+    logger.info(f"Inflating LVol: {lvol.get_id()}")
     snode = db_controller.get_storage_node_by_id(lvol.node_id)
 
     # creating RPCClient instance
