@@ -5,13 +5,14 @@ import logging as lg
 import time
 import uuid
 
-from simplyblock_core.controllers import lvol_controller, snapshot_events
+from simplyblock_core.controllers import lvol_controller, snapshot_events, pool_controller
 
 from simplyblock_core import utils, distr_controller, constants
-from simplyblock_core.kv_store import DBController
+from simplyblock_core.db_controller import DBController
 from simplyblock_core.models.pool import Pool
 from simplyblock_core.models.snapshot import SnapShot
 from simplyblock_core.models.lvol_model import LVol
+from simplyblock_core.models.storage_node import StorageNode
 from simplyblock_core.rpc_client import RPCClient
 from simplyblock_core.snode_client import SNodeClient
 
@@ -44,9 +45,14 @@ def add(lvol_id, snapshot_name):
             logger.error(msg)
             return False, msg
 
-    logger.info(f"Creating snapshot: {snapshot_name} from LVol: {lvol.id}")
+    logger.info(f"Creating snapshot: {snapshot_name} from LVol: {lvol.get_id()}")
     snode = db_controller.get_storage_node_by_id(lvol.node_id)
 
+    if snode.status != StorageNode.STATUS_ONLINE:
+        msg = f"Node is not online, {snode.status}"
+        logger.error(msg)
+        return False, msg
+
 ##############################################################################
     # Validate adding snap on storage node
     snode_api = SNodeClient(snode.api_endpoint)
@@ -68,19 +74,53 @@ def add(lvol_id, snapshot_name):
         logger.error(error)
         return False
 
+    if pool.pool_max_size > 0:
+        total = pool_controller.get_pool_total_capacity(pool.get_id())
+        if total + lvol.size > pool.pool_max_size:
+            msg = f"Pool max size has reached {utils.humanbytes(total)} of {utils.humanbytes(pool.pool_max_size)}"
+            logger.error(msg)
+            return False, msg
+
+    if snode.max_snap:
+        cnt = db_controller.get_snapshots_by_node_id(snode.get_id())
+        if cnt and len(cnt)+1 > snode.max_snap:
+            msg = f"Storage node snapshots count must be less than max_snap:{snode.max_snap}"
+            logger.error(msg)
+            return False, msg
+
 ##############################################################################
 
     rpc_client = RPCClient(snode.mgmt_ip, snode.rpc_port, snode.rpc_username, snode.rpc_password)
-    spdk_mem_info_before = rpc_client.ultra21_util_get_malloc_stats()
 
+    blobid = 0
+    snap_uuid = ""
+    rpc_client.bdev_lvol_set_leader(True, lvs_name=lvol.lvs_name)
     logger.info("Creating Snapshot bdev")
     ret = rpc_client.lvol_create_snapshot(f"{lvol.lvs_name}/{lvol.lvol_bdev}", snapshot_name)
     if not ret:
         return False, "Failed to create snapshot bdev"
 
-##############################################################################
+    size = lvol.size
+    snap_bdev = rpc_client.get_bdevs(f"{lvol.lvs_name}/{snapshot_name}")
+    if snap_bdev:
+        snap_uuid = snap_bdev[0]['uuid']
+        blobid = snap_bdev[0]['driver_specific']['lvol']['blobid']
+        size = snap_bdev[0]['driver_specific']['lvol']['num_allocated_clusters'] * lvol.cluster_size
+
+    if snode.secondary_node_id and blobid:
+        sec_node = db_controller.get_storage_node_by_id(snode.secondary_node_id)
+        if sec_node.status == StorageNode.STATUS_ONLINE:
+            sec_rpc_client = RPCClient(sec_node.mgmt_ip, sec_node.rpc_port, sec_node.rpc_username, sec_node.rpc_password)
+            sec_rpc_client.bdev_lvol_set_leader(False, lvs_name=lvol.lvs_name)
+            ret = sec_rpc_client.bdev_lvol_snapshot_register(
+                f"{lvol.lvs_name}/{lvol.lvol_bdev}", snapshot_name, snap_uuid, blobid)
+
+
     snap = SnapShot()
     snap.uuid = str(uuid.uuid4())
+    snap.snap_uuid = snap_uuid
+    snap.size = size
+    snap.blobid = blobid
     snap.pool_uuid = pool.get_id()
     snap.cluster_id = pool.cluster_id
     snap.snap_name = snapshot_name
@@ -88,15 +128,6 @@ def add(lvol_id, snapshot_name):
     snap.created_at = int(time.time())
     snap.lvol = lvol
 
-    spdk_mem_info_after = rpc_client.ultra21_util_get_malloc_stats()
-    logger.debug("ultra21_util_get_malloc_stats:")
-    logger.debug(spdk_mem_info_after)
-    diff = {}
-    for key in spdk_mem_info_after.keys():
-        diff[key] = spdk_mem_info_after[key] - spdk_mem_info_before[key]
-    logger.info("spdk mem diff:")
-    logger.info(json.dumps(diff, indent=2))
-    snap.mem_diff = diff
     snap.write_to_db(db_controller.kv_store)
 
     if lvol.cloned_from_snap:
@@ -124,7 +155,7 @@ def list(all=False):
         data.append({
             "UUID": snap.uuid,
             "Name": snap.snap_name,
-            "Size": utils.humanbytes(snap.lvol.size),
+            "Size": utils.humanbytes(snap.size),
             "BDev": snap.snap_bdev,
             "LVol ID": snap.lvol.get_id(),
             "Created At": time.strftime("%H:%M:%S, %d/%m/%Y", time.gmtime(snap.created_at)),
@@ -133,7 +164,7 @@ def list(all=False):
     return utils.print_table(data)
 
 
-def delete(snapshot_uuid, force=False):
+def delete(snapshot_uuid, force_delete=False):
     snap = db_controller.get_snapshot_by_id(snapshot_uuid)
     if not snap:
         logger.error(f"Snapshot not found {snapshot_uuid}")
@@ -144,17 +175,20 @@ def delete(snapshot_uuid, force=False):
         logger.error(f"Storage node not found {snap.lvol.node_id}")
         return False
 
+    clones = []
     for lvol in db_controller.get_lvols(snode.cluster_id):
         if lvol.cloned_from_snap and lvol.cloned_from_snap == snapshot_uuid:
-            logger.warning(f"Soft delete snapshot with clones, lvol ID: {lvol.get_id()}")
-            snap.deleted = True
-            snap.write_to_db(db_controller.kv_store)
-            return True
+            clones.append(lvol)
+
+    if len(clones) > 1:
+        logger.warning(f"Soft delete snapshot with clones")
+        snap = db_controller.get_snapshot_by_id(snapshot_uuid)
+        snap.deleted = True
+        snap.write_to_db(db_controller.kv_store)
+        return True
 
     logger.info(f"Removing snapshot: {snapshot_uuid}")
 
-    snode = db_controller.get_storage_node_by_id(snap.lvol.node_id)
-
     # creating RPCClient instance
     rpc_client = RPCClient(
         snode.mgmt_ip,
@@ -162,11 +196,38 @@ def delete(snapshot_uuid, force=False):
         snode.rpc_username,
         snode.rpc_password)
 
+    ret = rpc_client.bdev_lvol_set_leader(True, lvs_name=snap.lvol.lvs_name)
+    if not ret:
+        logger.error(f"Failed to set leader for primary node: {snode.get_id()}")
+        if not force_delete:
+            return False
+
     ret = rpc_client.delete_lvol(snap.snap_bdev)
     if not ret:
-        logger.error(f"Failed to delete BDev {snap.snap_bdev}")
-        if not force:
+        logger.error(f"Failed to delete snap from node: {snode.get_id()}")
+        if not force_delete:
             return False
+
+    if snode.secondary_node_id:
+        sec_node = db_controller.get_storage_node_by_id(snode.secondary_node_id)
+        if sec_node.status == StorageNode.STATUS_ONLINE:
+            sec_rpc_client = RPCClient(
+                sec_node.mgmt_ip,
+                sec_node.rpc_port,
+                sec_node.rpc_username,
+                sec_node.rpc_password)
+
+            ret = sec_rpc_client.bdev_lvol_set_leader(False, lvs_name=snap.lvol.lvs_name)
+            if not ret:
+                logger.error(f"Failed to set leader for secondary node: {sec_node.get_id()}")
+                if not force_delete:
+                    return False
+            ret = sec_rpc_client.delete_lvol(snap.snap_bdev)
+            if not ret:
+                logger.error(f"Failed to delete snap from node: {sec_node.get_id()}")
+                if not force_delete:
+                    return False
+
     snap.remove(db_controller.kv_store)
 
     base_lvol = db_controller.get_lvol_by_id(snap.lvol.get_id())
@@ -230,12 +291,27 @@ def clone(snapshot_id, clone_name, new_size=0):
     memory_free = result["memory_details"]["free"]
     huge_free = result["memory_details"]["huge_free"]
     total_node_capacity = db_controller.get_snode_size(snode.get_id())
+    lvols = db_controller.get_lvols_by_node_id(snode.get_id())
     error = utils.validate_add_lvol_or_snap_on_node(
-        memory_free, huge_free, snode.max_lvol, snap.lvol.size,  total_node_capacity, len(snode.lvols))
+        memory_free, huge_free, snode.max_lvol, snap.lvol.size,  total_node_capacity, len(lvols))
     if error:
         logger.error(error)
         return False, f"Failed to add lvol on node {snode.get_id()}"
 
+    if pool.pool_max_size > 0:
+        total = pool_controller.get_pool_total_capacity(pool.get_id())
+        if total + snap.lvol.size > pool.pool_max_size:
+            msg = f"Pool max size has reached {utils.humanbytes(total)} of {utils.humanbytes(pool.pool_max_size)}"
+            logger.error(msg)
+            return False, msg
+
+    if snode.max_snap:
+        cnt = db_controller.get_snapshots_by_node_id(snode.get_id())
+        if cnt and len(cnt)+1 > snode.max_lvol:
+            msg = f"Storage node LVol count must be less than max_lvol:{snode.max_lvol}"
+            logger.error(msg)
+            return False, msg
+
     cluster = db_controller.get_cluster_by_id(snode.cluster_id)
 
     lvol = LVol()
@@ -253,7 +329,7 @@ def clone(snapshot_id, clone_name, new_size=0):
     lvol.mode = 'read-write'
     lvol.cloned_from_snap = snapshot_id
     lvol.nqn = cluster.nqn + ":lvol:" + lvol.uuid
-    lvol.pool_uuid = pool.id
+    lvol.pool_uuid = pool.get_id()
     lvol.ha_type = snap.lvol.ha_type
     lvol.lvol_type = 'lvol'
     lvol.ndcs = snap.lvol.ndcs
@@ -263,80 +339,91 @@ def clone(snapshot_id, clone_name, new_size=0):
     lvol.distr_page_size = snap.lvol.distr_page_size
     lvol.guid = lvol_controller._generate_hex_string(16)
     lvol.vuid = snap.lvol.vuid
+    lvol.snapshot_name = snap.snap_bdev
 
     lvol.status = LVol.STATUS_ONLINE
-    # lvol.bdev_stack = snap.lvol.bdev_stack
     lvol.bdev_stack = [
         {
             "type": "bdev_lvol_clone",
-            "name": lvol.top_bdev
+            "name": lvol.top_bdev,
+            "params": {
+                "snapshot_name": lvol.snapshot_name,
+                "clone_name": lvol.lvol_bdev
+            }
         }
     ]
 
-    # if new_size:
-    #     if snap.lvol.size >= new_size:
-    #         msg = f"New size {new_size} must be higher than the original size {snap.lvol.size}"
-    #         logger.error(msg)
-    #         return False, msg
-    #
-    #     if snap.lvol.max_size < new_size:
-    #         msg = f"New size {new_size} must be smaller than the max size {snap.lvol.max_size}"
-    #         logger.error(msg)
-    #         return False, msg
-    #     lvol.size = new_size
-
-    rpc_client = RPCClient(snode.mgmt_ip, snode.rpc_port, snode.rpc_username, snode.rpc_password)
-    spdk_mem_info_before = rpc_client.ultra21_util_get_malloc_stats()
-
-    logger.info("Creating clone bdev")
-    ret = rpc_client.lvol_clone(snap.snap_bdev, lvol.lvol_bdev)
-    if not ret:
-        return False, "Failed to create clone lvol bdev"
-
-    if "crypto" in snap.lvol.lvol_type:
-        lvol.crypto_bdev = f"crypto_{lvol.lvol_name}"
+    if snap.lvol.crypto_bdev:
+        lvol.crypto_bdev = f"crypto_{lvol.lvol_bdev}"
+        lvol.bdev_stack.append({
+            "type": "crypto",
+            "name": lvol.crypto_bdev,
+            "params": {
+                "name": lvol.crypto_bdev,
+                "base_name": lvol.top_bdev,
+                "key1": snap.lvol.crypto_key1,
+                "key2": snap.lvol.crypto_key2,
+            }
+        })
+        lvol.lvol_type += ',crypto'
+        lvol.top_bdev = lvol.crypto_bdev
         lvol.crypto_key1 = snap.lvol.crypto_key1
         lvol.crypto_key2 = snap.lvol.crypto_key2
 
-        ret = lvol_controller._create_crypto_lvol(
-            rpc_client, lvol.crypto_bdev, lvol.top_bdev, lvol.crypto_key1, lvol.crypto_key2)
-        if not ret:
-            return False, "Failed to create clone lvol bdev"
+    if new_size:
+        if snap.lvol.size >= new_size:
+            msg = f"New size {new_size} must be higher than the original size {snap.lvol.size}"
+            logger.error(msg)
+            return False, msg
 
-        lvol.lvol_type += ',crypto'
-        lvol.top_bdev = lvol.crypto_bdev
+        if snap.lvol.max_size < new_size:
+            msg = f"New size {new_size} must be smaller than the max size {snap.lvol.max_size}"
+            logger.error(msg)
+            return False, msg
+        lvol.size = new_size
 
-    subsystem_nqn = lvol.nqn
-    logger.info("creating subsystem %s", subsystem_nqn)
-    ret = rpc_client.subsystem_create(subsystem_nqn, 'sbcli-cn', lvol.uuid)
+    rpc_client = RPCClient(snode.mgmt_ip, snode.rpc_port, snode.rpc_username, snode.rpc_password)
 
-    # add listeners
-    logger.info("adding listeners")
-    for iface in snode.data_nics:
-        if iface.ip4_address:
-            tr_type = iface.get_transport_type()
-            logger.info("adding listener for %s on IP %s" % (subsystem_nqn, iface.ip4_address))
-            ret = rpc_client.listeners_create(subsystem_nqn, tr_type, iface.ip4_address, "4420")
+    if snap.lvol.ha_type == 'single':
+        ret, error = lvol_controller.add_lvol_on_node(lvol, snode)
+        if error:
+            return ret, error
+        lvol.nodes = [snode.get_id()]
+    elif snap.lvol.ha_type == "ha":
+        lvol_bdev, error = lvol_controller.add_lvol_on_node(lvol, snode)
+        if error:
+            return False, error
 
-    logger.info(f"add lvol {clone_name} to subsystem")
-    ret = rpc_client.nvmf_subsystem_add_ns(subsystem_nqn, lvol.top_bdev, lvol.uuid, lvol.guid)
-    if not ret:
-        return False, "Failed to add bdev to subsystem"
+        lvol.lvol_uuid = lvol_bdev['uuid']
+        lvol.blobid = lvol_bdev['driver_specific']['lvol']['blobid']
 
-    spdk_mem_info_after = rpc_client.ultra21_util_get_malloc_stats()
-    diff = {}
-    for key in spdk_mem_info_after.keys():
-        diff[key] = spdk_mem_info_after[key] - spdk_mem_info_before[key]
+        sec_node = db_controller.get_storage_node_by_id(snode.secondary_node_id)
+        if sec_node.status == StorageNode.STATUS_ONLINE:
+            ret, error = lvol_controller.add_lvol_on_node(lvol, sec_node, ha_inode_self=1)
+            if error:
+                return ret, error
 
-    logger.info("spdk mem diff:")
-    logger.info(json.dumps(diff, indent=2))
-    lvol.mem_diff = diff
-    lvol.write_to_db(db_controller.kv_store)
+        nodes_ids = [
+            snode.get_id(),
+            snode.secondary_node_id]
+        lvol.nodes = nodes_ids
+##########################################################################################################
+    # spdk_mem_info_after = rpc_client.ultra21_util_get_malloc_stats()
+    # diff = {}
+    # for key in spdk_mem_info_after.keys():
+    #     diff[key] = spdk_mem_info_after[key] - spdk_mem_info_before[key]
+    #
+    # logger.info("spdk mem diff:")
+    # logger.info(json.dumps(diff, indent=2))
+    # lvol.mem_diff = diff
+    # lvol.write_to_db(db_controller.kv_store)
 
-    pool.lvols.append(lvol.uuid)
-    pool.write_to_db(db_controller.kv_store)
-    snode.lvols.append(lvol.uuid)
-    snode.write_to_db(db_controller.kv_store)
+    # pool = db_controller.get_pool_by_id(snap.lvol.pool_uuid)
+    # pool.lvols += 1
+    # pool.write_to_db(db_controller.kv_store)
+    # snode = db_controller.get_storage_node_by_id(snode.get_id())
+    # snode.lvols += 1
+    # snode.write_to_db(db_controller.kv_store)
 
     if snap.snap_ref_id:
         ref_snap = db_controller.get_snapshot_by_id(snap.snap_ref_id)
@@ -348,6 +435,6 @@ def clone(snapshot_id, clone_name, new_size=0):
 
     logger.info("Done")
     snapshot_events.snapshot_clone(snap, lvol)
-    # if new_size:
-    #     lvol_controller.resize_lvol(lvol.get_id(), new_size)
+    if new_size:
+        lvol_controller.resize_lvol(lvol.get_id(), new_size)
     return True, lvol.uuid
