@@ -7,6 +7,7 @@ from simplyblock_core.models.lvol_model import LVol
 class SnapShot(BaseModel):
 
     base_bdev: str = ""
+    blobid: int = 0
     cluster_id: str = ""
     created_at: int = 0
     health_check: bool = True
@@ -14,6 +15,8 @@ class SnapShot(BaseModel):
     mem_diff: dict = {}
     pool_uuid: str = ""
     ref_count: int = 0
+    size: int = 0
     snap_bdev: str = ""
     snap_name: str = ""
     snap_ref_id: str = ""
+    snap_uuid: str = ""
