@@ -2,7 +2,7 @@
 import time
 from datetime import datetime
 
-from simplyblock_core import constants, kv_store, utils
+from simplyblock_core import constants, db_controller, utils
 from simplyblock_core.controllers import tasks_events, tasks_controller
 from simplyblock_core.models.job_schedule import JobSchedule
 
@@ -33,7 +33,6 @@ def task_runner(task):
         task.status = JobSchedule.STATUS_RUNNING
         task.write_to_db(db_controller.kv_store)
         tasks_events.task_updated(task)
-        time.sleep(30)
 
     if snode.status != StorageNode.STATUS_ONLINE:
         task.function_result = "node is not online, retrying"
@@ -70,7 +69,7 @@ def task_runner(task):
             task.write_to_db(db_controller.kv_store)
             return False
 
-        device = db_controller.get_storage_devices(task.device_id)
+        device = db_controller.get_storage_device_by_id(task.device_id)
         distr_name = task.function_params["distr_name"]
 
         if not device:
@@ -79,7 +78,10 @@ def task_runner(task):
             task.write_to_db(db_controller.kv_store)
             return True
 
-        rsp = rpc_client.distr_migration_to_primary_start(device.cluster_device_order, distr_name)
+        qos_high_priority = False
+        if db_controller.get_cluster_by_id(snode.cluster_id).enable_qos:
+            qos_high_priority = True
+        rsp = rpc_client.distr_migration_to_primary_start(device.cluster_device_order, distr_name, qos_high_priority)
         if not rsp:
             logger.error(f"Failed to start device migration task, storage_ID: {device.cluster_device_order}")
             task.function_result = "Failed to start device migration task"
@@ -94,34 +96,19 @@ def task_runner(task):
     if "migration" in task.function_params:
         mig_info = task.function_params["migration"]
         res = rpc_client.distr_migration_status(**mig_info)
-        for st in res:
-            if st['status'] == "completed":
-                if st['error'] != 0:
-                    task.function_result = "mig completed with errors, retrying"
-                    task.retry += 1
-                    task.status = JobSchedule.STATUS_SUSPENDED
-                    del task.function_params['migration']
-                else:
-                    task.status = JobSchedule.STATUS_DONE
-                    task.function_result = "Done"
-
-                task.write_to_db(db_controller.kv_store)
-                return True
-
-            task.function_result = f"progress: {st['progress']}%, errors: {st['error'] }"
-            task.write_to_db(db_controller.kv_store)
-
-    task.retry += 1
-    task.write_to_db(db_controller.kv_store)
-    return False
+        return utils.handle_task_result(task, res)
+    else:
+        task.retry += 1
+        task.write_to_db(db_controller.kv_store)
+        return False
 
 
 # get DB controller
-db_controller = kv_store.DBController()
+db_controller = db_controller.DBController()
 
 logger.info("Starting Tasks runner...")
 while True:
-    time.sleep(30)
+    time.sleep(3)
     clusters = db_controller.get_clusters()
     if not clusters:
         logger.error("No clusters found!")
@@ -129,7 +116,7 @@ while True:
         for cl in clusters:
             tasks = db_controller.get_job_tasks(cl.get_id(), reverse=False)
             for task in tasks:
-                delay_seconds = 10
+                delay_seconds = 3
                 if task.function_name == JobSchedule.FN_DEV_MIG:
                     if task.status in [JobSchedule.STATUS_NEW, JobSchedule.STATUS_SUSPENDED]:
                         active_task = tasks_controller.get_active_node_mig_task(task.cluster_id, task.node_id)
