@@ -15,9 +15,7 @@ from graypy import GELFTCPHandler
 
 from simplyblock_core import constants
 from simplyblock_core import shell_utils
-
-
-logger = logging.getLogger()
+from simplyblock_core.models.job_schedule import JobSchedule
 
 
 def get_env_var(name, default=None, is_required=False):
@@ -126,12 +124,12 @@ def generate_string(length):
 
 
 def get_docker_client(cluster_id=None):
-    from simplyblock_core.kv_store import DBController
+    from simplyblock_core.db_controller import DBController
     db_controller = DBController()
-    nodes = db_controller.get_mgmt_nodes(cluster_id)
+    nodes = db_controller.get_mgmt_nodes()
     if not nodes:
         logger.error("No mgmt nodes was found in the cluster!")
-        exit(1)
+        return False
 
     docker_ips = [node.docker_ip_port for node in nodes]
 
@@ -139,9 +137,10 @@ def get_docker_client(cluster_id=None):
         try:
             c = docker.DockerClient(base_url=f"tcp://{ip}", version="auto")
             return c
-        except docker.errors.DockerException as e:
+        except Exception as e:
             print(e)
-    raise e
+            raise e
+    return False
 
 
 def dict_agg(data, mean=False):
@@ -297,7 +296,7 @@ def sum_records(records):
 
 
 def get_random_vuid():
-    from simplyblock_core.kv_store import DBController
+    from simplyblock_core.db_controller import DBController
     db_controller = DBController()
     used_vuids = []
     nodes = db_controller.get_storage_nodes()
@@ -306,7 +305,7 @@ def get_random_vuid():
             type = bdev['type']
             if type == "bdev_distr":
                 vuid = bdev['params']['vuid']
-            elif type == "bdev_raid":
+            elif type == "bdev_raid"  and "jm_vuid" in bdev:
                 vuid = bdev['jm_vuid']
             else:
                 continue
@@ -504,9 +503,9 @@ def calculate_pool_count(alceml_count, number_of_distribs, cpu_count, poller_cou
     poller_number = poller_count if poller_count else cpu_count
     #small_pool_count = (3 + alceml_count + lvol_count + 2 * snap_count + 1) * 256 + poller_number * 127 + 384 + 128 * poller_number + constants.EXTRA_SMALL_POOL_COUNT
 
-    small_pool_count = (6 + alceml_count + number_of_distribs) * 256 + poller_number * 127 + 384 + 128 * poller_number + constants.EXTRA_SMALL_POOL_COUNT
+    small_pool_count = 384 * (alceml_count + number_of_distribs + 3 + poller_count) + (6 + alceml_count + number_of_distribs) * 256 + poller_number * 127 + 384 + 128 * poller_number + constants.EXTRA_SMALL_POOL_COUNT
     #large_pool_count = (3 + alceml_count + lvol_count + 2 * snap_count + 1) * 32 + poller_number * 15 + 384 + 16 * poller_number + constants.EXTRA_LARGE_POOL_COUNT
-    large_pool_count = (6 + alceml_count + number_of_distribs) * 32 + poller_number * 15 + 384 + 16 * poller_number + constants.EXTRA_LARGE_POOL_COUNT
+    large_pool_count = 48 * (alceml_count + number_of_distribs + 3 + poller_count) + (6 + alceml_count + number_of_distribs) * 32 + poller_number * 15 + 384 + 16 * poller_number + constants.EXTRA_LARGE_POOL_COUNT
     return small_pool_count, large_pool_count
 
 
@@ -578,25 +577,32 @@ def decimal_to_hex_power_of_2(decimal_number):
     return hex_result
 
 
-def get_logger(name):
+def get_logger(name=""):
     # first configure a root logger
-    logger = logging.getLogger()
+    logging.getLogger("urllib3.connectionpool").setLevel(logging.WARNING)
+    logg = logging.getLogger(f"root")
+
     log_level = os.getenv("SIMPLYBLOCK_LOG_LEVEL")
     log_level = log_level.upper() if log_level else constants.LOG_LEVEL
 
     try:
-        logger.setLevel(log_level)
+        logg.setLevel(log_level)
     except ValueError as e:
-        logger.warning(f'Invalid SIMPLYBLOCK_LOG_LEVEL: {str(e)}')
-        logger.setLevel(constants.LOG_LEVEL)
+        logg.warning(f'Invalid SIMPLYBLOCK_LOG_LEVEL: {str(e)}')
+        logg.setLevel(constants.LOG_LEVEL)
 
-    logger_handler = logging.StreamHandler(stream=sys.stdout)
-    logger_handler.setFormatter(logging.Formatter('%(asctime)s: %(levelname)s: %(message)s'))
-    logger.addHandler(logger_handler)
+    if not logg.hasHandlers():
+        logger_handler = logging.StreamHandler(stream=sys.stdout)
+        logger_handler.setFormatter(logging.Formatter('%(asctime)s: %(levelname)s: %(message)s'))
+        logg.addHandler(logger_handler)
+        gelf_handler = GELFTCPHandler('0.0.0.0', constants.GELF_PORT)
+        logg.addHandler(gelf_handler)
 
-    gelf_handler = GELFTCPHandler('0.0.0.0', constants.GELF_PORT)
-    logger.addHandler(gelf_handler)
-    return logging.getLogger(name)
+    if name:
+        logg = logging.getLogger(f"root.{name}")
+        logg.propagate = True
+
+    return logg
 
 
 def parse_size(size_string: str):
@@ -653,4 +659,42 @@ def strfdelta(tdelta):
             if values[field] > 0:
                 out += f"{values[field]}{field.lower()} "
 
-    return out.strip()
\ No newline at end of file
+    return out.strip()
+
+
+def handle_task_result(task: JobSchedule, res: dict, allowed_error_codes = None):
+    if res:
+        res_data = res[0]
+        migration_status = res_data.get("status")
+        error_code = res_data.get("error", -1)
+        progress = res_data.get("progress", -1)
+        if migration_status == "completed":
+            if error_code == 0:
+                task.function_result = "Done"
+                task.status = JobSchedule.STATUS_DONE
+            elif error_code in allowed_error_codes:
+                task.function_result = f"mig completed with status: {error_code}"
+                task.status = JobSchedule.STATUS_DONE
+            else:
+                task.function_result = f"mig error: {error_code}, retrying"
+                task.retry += 1
+                task.status = JobSchedule.STATUS_SUSPENDED
+                del task.function_params['migration']
+
+            task.write_to_db()
+            return True
+
+        elif migration_status == "failed":
+            task.status = JobSchedule.STATUS_DONE
+            task.function_result = migration_status
+            task.write_to_db()
+            return True
+
+        else:
+            task.function_result = f"Status: {migration_status}, progress:{progress}"
+            task.write_to_db()
+    else:
+        logger.error("Failed to get mig status")
+
+
+logger = get_logger(__name__)
